package com.muyu.group.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.group.team.service.ActivityTeamProductSkuInfoService;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.group.team.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Optional;

@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo>
        implements ActivityTeamProductSkuInfoService {
    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;
    @Autowired
    private ProjectSkuCache projectSkuCache;

    /**
     * 通过团购活动ID获取团购中最优惠的价格
     *
     * @param teamId 团购ID
     * @return 优惠价格
     */
    @Override
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);

        // 优惠模型集合
        Optional<TeamProductDiscountPriceModel> discountPriceModelOptional = teamProductSkuInfoList.stream()
                .map(activityTeamProductSkuInfo -> {
                    ProjectSkuInfo projectSkuInfo = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                    return TeamProductDiscountPriceModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
                }).min((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue());
        if (discountPriceModelOptional.isEmpty()){
            throw new ServiceException("团购活动下没有商品绑定");
        }
        return discountPriceModelOptional.get();
    }

    /**
     * 通过活动ID获取 剩余库存
     *
     * @param teamId 活动ID
     * @return 库存
     */
    @Override
    public TeamProductStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L, Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L, Long::sum))
                .build();
    }

    /**
     * 添加团购商品SKU
     *
     * @param activityTeamProductSkuAddModelList 团购商品SKU添加模型集合
     */
    @Override
    public void batchSave(List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddModelList) {
        if (activityTeamProductSkuAddModelList == null || activityTeamProductSkuAddModelList.isEmpty()) {
            throw new ServiceException("入参添加商品规格模型，不可为空");
        }
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = activityTeamProductSkuAddModelList.stream().map(ActivityTeamProductSkuInfo::modelBuild).toList();
        this.saveBatch(teamProductSkuInfoList);
    }
    /**
     * 根据团购活动id查找商品sku
     * @param id
     * @return
     */
    @Override
    public List<ActivityTeamSkuInfoDetailModel> findByListSkuInfo(Long id) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,id);
        List<ActivityTeamProductSkuInfo> productSkuInfoList = activityTeamProductSkuInfoService.list(queryWrapper);
        List<ActivityTeamSkuInfoDetailModel> detailModelList = productSkuInfoList.stream().map(activityTeamProductSkuInfo -> {
            return ActivityTeamSkuInfoDetailModel.productSkuInfoEachBuild(activityTeamProductSkuInfo);
        }).toList();
        return detailModelList;
    }

    /**
     * 修改团购商品sku表
     * @param activityTeamProductSkuEditModelList
     */
    @Override
    public void editSku(List<ActivityTeamProductSkuEditModel> activityTeamProductSkuEditModelList) {
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfoList = activityTeamProductSkuEditModelList.stream().map(activityTeamProductSkuEditModel -> {
            return ActivityTeamProductSkuInfo.skuInfoBuild(activityTeamProductSkuEditModel);
        }).toList();
        this.updateBatchById(activityTeamProductSkuInfoList);
    }
    /**
     * 批量修改团购商品sku
     * @param activityTeamProductSkuUpdModelList 团购商品sku模型
     */
    @Override
    public void batchUpd(List<ActivityTeamProductSkuUpdModel> activityTeamProductSkuUpdModelList) {
        List<ActivityTeamProductSkuInfo> productSkuInfoList = activityTeamProductSkuUpdModelList.stream().map(activityTeamProductSkuUpdModel -> {
            return ActivityTeamProductSkuInfo.skuBuild(activityTeamProductSkuUpdModel);
        }).toList();
        this.updateBatchById(productSkuInfoList);
    }
    /**
     * 批量添加团购sku
     * @param activityTeamSkuAddModelCollection
     */
    @Override
    public void batchAdd(Collection<ActivityTeamSkuAddModel> activityTeamSkuAddModelCollection ) {
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfoList = activityTeamSkuAddModelCollection.stream().map(activityTeamSkuAddModel -> {
            return ActivityTeamProductSkuInfo.skuBuildModel(activityTeamSkuAddModel);
        }).toList();
        this.saveBatch(activityTeamProductSkuInfoList);
    }
    /**
     * 根据拼图活动id查找sku模型
     * @param id
     * @return
     */
    @Override
    public List<ActivityTeamProductSkuUpdModel> findByTeamId(long id) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,id);
        List<ActivityTeamProductSkuInfo> productSkuInfoList = activityTeamProductSkuInfoService.list(queryWrapper);
        List<ActivityTeamProductSkuUpdModel> modelList = productSkuInfoList.stream().map(activityTeamProductSkuInfo -> {
            return ActivityTeamProductSkuUpdModel.infolBuild(activityTeamProductSkuInfo);
        }).toList();
        return modelList;
    }
    /**
     * 批量删除团购sku
     * @param subtract
     */
    @Override
    public void batchDel(Collection<ActivityTeamProductSkuUpdModel> subtract) {
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfoList = subtract.stream().map(activityTeamProductSkuUpdModel -> {
            return ActivityTeamProductSkuInfo.skuModel(activityTeamProductSkuUpdModel);
        }).toList();
        this.removeBatchByIds(activityTeamProductSkuInfoList);
    }

    /**
     * 根据活动id查找商品sku
     * @param id
     * @return
     */
    @Override
    public List<ActivityTeamSkuAddModel> activityTeamAdd(long id) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,id);
        List<ActivityTeamProductSkuInfo> productSkuInfoList = activityTeamProductSkuInfoService.list(queryWrapper);
        List<ActivityTeamSkuAddModel> activityTeamSkuAddModelList = productSkuInfoList.stream().map(activityTeamProductSkuInfo -> {
            return ActivityTeamSkuAddModel.infolBuild(activityTeamProductSkuInfo, () -> activityTeamProductSkuInfo.getProductId());
        }).toList();
        return activityTeamSkuAddModelList;
    }
}
